
"""
File for constant TUIO values
"""
TUIO_CURSOR = "/tuio/2Dcur"
TUIO_OBJECT = "/tuio/2Dobj"
TUIO_BLOB   = "/tuio/2Dblb"

TUIO_ALIVE = "alive"
TUIO_SET =  "set"
TUIO_END = "fseq"
TUIO_SOURCE = "source"
